#!/usr/bin/python pseudoserver.py
#psm-bopm.py
# module for pypseudoserver
# written by radicand
# doesn't do much by itself

#todo make use of the debug flag i have in cfg

from twisted.internet import threads, reactor
import socket, re, struct
from psmodule import *

class PSModule_bopm(PSModule):
	BOPM_LIST = []
	BOPM_TOTAL = 0
	BOPM_BANNED = 0
	VERSION = 4
	
	uid = ""
	sourceip = ""
	destip = ""
	destport = ""
	
	def reload(self):
		try:
			PSModule.reload(self)
			self.BOPM_LIST = [(int(port), self.config.get('bopm-ports', port).split(","))
				for port in self.config.options('bopm-ports')]
			return True
		except Exception, err:
			self.logger.log("Error reloading bomp lists: %s" % err, L_ERROR)
			return False
	
	def startup(self):
		if not PSModule.startup(self): return False
		if not self.reload(): raise("Error loading bomp lists")
			
		try:
			self.sourceip = self.config.get('bopm', 'sourceip')
			self.sourcehost = self.config.get('bopm', 'sourcehost')
			self.destip = self.config.get('bopm', 'destip')
			self.destport = self.config.getint('bopm', 'destport')
			self.watermark = self.config.getint('bopm', 'watermark')
		except Exception, err:
			self.logger.log("Error setting BOPM module options, are they defined in config? (%s)" % err, L_ERROR)
			raise Exception("Config options are invalid for BOPM module")
		
		try:
			self.uid, user = self.parent.createFakeUser(
				self.config.get('bopm', 'nick'),
				self.config.get('bopm', 'user'),
				self.config.get('bopm', 'host'),
				self.config.get('bopm', 'gecos'),
				self.config.get('bopm', 'modes'),
				self.config.get('bopm', 'nspass'),
				version="PyPsd Blacklist Open Proxy Monitor v%d" % self.VERSION
			)
		except Exception, err:
			self.logger.log("Error creating user for BOPM module, do the options exist in config? (%s)" % err, L_ERROR)
			raise Exception("BOPM module has invalid user parameters in config")
			
		try:
			num = self.dbp.execute("SELECT num FROM record WHERE name='bopm-total';")
			if num:
				self.BOPM_TOTAL = self.dbp.fetchone()[0]

			num = self.dbp.execute("SELECT num FROM record WHERE name='bopm-found';")
			if num:
				self.BOPM_BANNED = self.dbp.fetchone()[0]

		except Exception, err:
			self.logger.log("Unable to fetch historical records for bopm: %s" % err, L_ERROR)
	
		return True
	
	def shutdown(self):
		PSModule.shutdown(self)
		try:
			self.dbp.execute("INSERT INTO record (name, num) VALUES ('bopm-total', %s) ON DUPLICATE KEY UPDATE num=%s;", (self.BOPM_TOTAL,self.BOPM_TOTAL))
			self.dbp.execute("INSERT INTO record (name, num) VALUES ('bopm-found', %s) ON DUPLICATE KEY UPDATE num=%s;", (self.BOPM_BANNED,self.BOPM_BANNED))
		except Exception, err:
			self.logger.log("Error updating historical records for bopm module: %s" % err, L_ERROR)
		
		self.parent.quitFakeUser(self.uid)

	def isIP(self, ip):
		"""Checks the supplied argument to see if it validates as an IP address"""
		
		if re.match("^(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$", ip):
			return True
		else: return False
		
## Begin event hooks	
	def bopm_CHECK(self, ip):
		"""Check host for vulnerable proxy stuff
		Param is the ip to check
		"""
		
		scan = None
		socket.setdefaulttimeout(2)
		self.logger.log("Using list...%s" % self.BOPM_LIST, L_DEBUG)

		q = len(reactor.threadpool.q.queue)
		if q > self.watermark:
			self.logger.log("Passing on BOPM for %s, wm=%d < q=%d" % (ip, self.watermark, q), L_ERROR) #update this later
			return
		
		def sendRecv(message, term=None):
			"""Send a message and read until we see 'term'"""
			
			totalsent = 0
			tosend = len(message)
			while totalsent < tosend:
				sent = scan.send(message[totalsent:])
				if not sent:
					raise RuntimeError, "socket broken"
				totalsent += sent
			if term:
				#get first few bytes
				msg = ""
				while msg.find(term) == -1: #gets first line
					chunk = scan.recv(512)
					if not chunk:
						raise RuntimeError, "socket broken"
					msg += chunk
				return msg
			else: return totalsent
		
		def listenFor(term):
			"""Keep reading until we see 'term'"""
			
			msg = ""
			while msg.find(term) == -1: #gets first line
				chunk = scan.recv(512)
				if not chunk:
					raise RuntimeError, "socket broken"
				msg += chunk
			return msg	
		
		def cleanup():
			"""Close down the socket with no uncertain terms"""
			
			if not scan: return
			try:
				scan.shutdown(socket.SHUT_RDWR)
				scan.close()
			except:
				pass
		
		self.logger.log("Scanning %s..." % ip, L_DEBUG)
		for port, methods in self.BOPM_LIST:
			for method in methods:
				try:
					try:
						self.logger.log("Checking: %d, %s" % (port, method), L_DEBUG)
						cleanup()
						scan = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
						scan.bind((self.sourceip,0))
						scan.connect((ip, port))
						self.logger.log("%d is open on %s, scanning for proxy..." % (port, ip), L_DEBUG)
					except socket.timeout:
						cleanup()
						self.logger.log("SCAN_RESULT: Port %d was closed" % (port), L_DEBUG)
						break #if port is closed, dont try again!
					except socket.error, (value,message):
						cleanup()
						self.logger.log("SCAN_RESULT: Port %d was closed" % (port), L_DEBUG)
						break #if port is closed, dont try again!
				except Exception, err:
					#something seriously bad happened
					self.logger.log("Error [CRIT] in bopm checker: %s" % err, L_ERROR)
					return False
				try:
					if method == "HTTP":
						try:
							msg = sendRecv("CONNECT %s:%d HTTP/1.0\r\n\r\n" %
								(self.destip, self.destport), "\r\n")
							self.logger.log(msg, L_DEBUG)
						except Exception, err:
							cleanup()
							self.logger.log("SCAN_RESULT: Port %d was not HTTP" % (port), L_DEBUG)
							continue
						if msg.startswith("HTTP/1.0 200 Conn"):
							raise Exception("taint found")
					elif method == "ROUTER":
						try:
							msg = listenFor("Password:")
							self.logger.log(msg, L_DEBUG)
							# should be valid, since we listen for Password:
							msg2 = sendRecv("cisco\r\n", ">")
							self.logger.log(msg2, L_DEBUG)
						except Exception, err:
							cleanup()
							self.logger.log("SCAN_RESULT: Port %d was not ROUTER" % (port), L_DEBUG)
							continue
						raise Exception("taint found")
					elif method == "SOCKS4":
						try:
							msg = sendRecv("\x04\x01" +
								struct.pack(">H", self.destport) +
								socket.inet_aton(self.destip) +
								"\x00", "\x00")
							self.logger.log(msg, L_DEBUG)
						except Exception, err:
							cleanup()
							self.logger.log("SCAN_RESULT: Port %d was not SOCKS4" % (port), L_DEBUG)
							continue
						if msg.find("\x5a") != -1 or msg.find("\x5c") != -1:
							#accepted or denied b/c no ident
							raise Exception("taint found")
					elif method == "SOCKS5":
						try:
							msg = sendRecv("\x05\x01\x00", "\x00")
							self.logger.log(msg, L_DEBUG)
						except Exception, err:
							cleanup()
							self.logger.log("SCAN_RESULT: Port %d was not SOCKS5" % (port), L_DEBUG)
							continue
						if msg.find("\x05") != -1:
							#accepted or denied b/c no ident
							raise Exception("taint found")
					# cleanup and continue loop
					self.logger.log("SCAN_RESULT: Port %d was not tainted" % (port), L_DEBUG)
					cleanup()
				except socket.timeout:
					# cleanup and continue loop
					self.logger.log("SCAN_RESULT: Port %d was not tainted" % (port), L_DEBUG)
					cleanup()
				except Exception, err:
					self.BOPM_BANNED += 1
					self.logger.log("SCAN_RESULT: Port %d was tainted" % (port), L_DEBUG)
					cleanup()
					return (port, method)
		return False

	def bopm_HANDLE(self, cb_list, user, type="auto", **kwargs):
		"""Callback for the main bopm checker function which
		is threaded for obvious reasons
		"""

		source = kwargs.get('source', self.logchan)
		
		if not cb_list:
			if type != "auto":
				self.parent.privMsg(source,
					"%s passed the manual BOPM scan." % user['ip'], self.uid)
			return
		elif not user: return
	
		self.parent.privMsg(source, "BOPM: Banned %s!%s@%s Match: %d/%s (#%d/%d)" %
			(user['nick'], user['user'], user['ip'],
			cb_list[0], cb_list[1], self.BOPM_BANNED, self.BOPM_TOTAL), self.uid)
		if not self.parent.get_user("GeoServ"): serv = "OperServ"
		else: serv = "GeoServ"
		self.parent.privMsg(serv, "akill add +3d *@%s Insecure proxy found, %s:%d - \
			Visit http://en.wikipedia.org/wiki/Open_proxy for more information." %
			(user['ip'], cb_list[1], cb_list[0]), self.uid)

		if "dnsbl_admin" in self.parent.modules:
			uid2, user2 = self.parent.get_user(self.uid)
			self.parent.modules['dnsbl_admin'].add(user['ip'], "1", "Open Proxy found by %s (%s!%s@%s has %s:%d insecured)" %
				(user2['nick'],user['nick'],user['user'],user['ip'],cb_list[1],cb_list[0]))
			
	def bopm_UID(self, prefix, params):
		"""Called when a user signs on
		Informs them of the US statute and triggers the thread to scan
		"""
		
		if self.parent.flood: return

		if not params[4].startswith("~"): # skip people who have ident
			self.logger.log("BOPM Skipping %s!%s@%s, ident is active" % (params[0], params[4], params[6]), L_DEBUG)
			return
		
		self.BOPM_TOTAL += 1

		#self.parent.privMsg(self.logchan, "BOPM: Checking %s!%s@%s (#%d)" %
		#	(params[0], params[4], params[6], self.BOPM_TOTAL),  self.uid)
		self.parent.notice(params[7],
			"Pursuant to United States Code (USC), Title 18, Part 1, Section 2701 Part (c),",
			self.uid)
		self.parent.notice(params[7],
			"%s will do a passive scan of your IP address to check for open proxies." %
			self.config.get('server', 'network'), self.uid)
		self.parent.notice(params[7],
			"If you do not consent to this scan, disconnect now. These scans are harmless and",
			self.uid)
		self.parent.notice(params[7],
			"will originate from %s (%s). For more information see #help." %
			(self.sourcehost, self.sourceip), self.uid)
		d = threads.deferToThread(self.bopm_CHECK, params[6])
		d.addCallback(self.bopm_HANDLE, self.parent.user_t[params[7]])
## End event hooks
		
## Begin command hooks	
	def cmd_nickscan(self, source, target, pieces):
		"""Scans the host associated with the supplied nickname"""
		
		if not pieces: return False
			
		uid, user = self.parent.get_user(pieces[0])
		if not user:
			self.parent.privMsg(target,
				"%s is not an online user, cannot scan" %
				(pieces[0]), self.uid)
			return True
		ip = user['ip']
		if not self.isIP(ip):
			self.parent.privMsg(target,
				"%s (%s) is spoofed or invalid, cannot scan" %
				(pieces[0], ip),
				self.uid)
			return True
		self.BOPM_TOTAL += 1
		self.parent.privMsg(target,
			"Initiating manual nick BOPM scan against %s/%s..." %
			(pieces[0], ip), self.uid)
		d = threads.deferToThread(self.bopm_CHECK, ip)
		d.addCallback(self.bopm_HANDLE, user,
			source=target, type="nick")
		return True
		
	def cmd_reload(self, source, target, pieces):
		"""Reloads bomp list
		TODO: need to move this out of the config.ini
		"""
		
		if self.reload():
			self.parent.privMsg(target,
				"BOPM reload successful", self.uid)
		else:
			self.parent.privMsg(target,
				"BOPM reload failed, check your config file", self.uid)
		return True
## End Command hooks

	def getCommands(self):
		return (('breload', {
			'master':None,
			'callback':self.cmd_reload,
			'usage':"-reloads the port and service list from the config file"}),
			('bnscan', {
			'channel':(self.logchan,),
			'callback':self.cmd_nickscan,
			'usage':"<nick> - issue a BOPM scan for nick"}),
		)


	def getHooks(self):
		return (('uid', self.bopm_UID),)

