#!/usr/bin/python pseudoserver.py
#psm-dnsbl.py
# module for pypseudoserver
# written by radicand
# doesn't do much by itself

from twisted.internet import threads
import socket
from psmodule import *

class PSModule_dnsbl(PSModule):
	DNSBL_LIST = []
	DNSBL_TOTAL = 0
	DNSBL_BANNED = 0
	VERSION = 2
	
	uid = ""
		
	def reload(self):
		try:
			PSModule.reload(self)
			self.DNSBL_LIST = [(service, self.config.get('dnsbl-lists', service))
				for service in self.config.options('dnsbl-lists')]
			return True
		except Exception, err:
			self.logger.log("Error reloading dnsbl lists: %s" % err, L_ERROR)
			return False
	
	def startup(self):
		if not PSModule.startup(self): return False
		if not self.reload(): raise("Error loading dnsbl lists")

		try:
			self.uid, user = self.parent.createFakeUser(
				self.config.get('dnsbl', 'nick'),
				self.config.get('dnsbl', 'user'),
				self.config.get('dnsbl', 'host'),
				self.config.get('dnsbl', 'gecos'),
				self.config.get('dnsbl', 'modes'),
				self.config.get('dnsbl', 'nspass'),
				version="PyPsd DNS Blacklist Checker v%d" % self.VERSION
			)
		except Exception, err:
			self.logger.log("Error creating user for dnsbl, is the config correct? (%s)" % err, L_ERROR)
			raise Exception("DNSBL module has invalid user information")

		try:
			num = self.dbp.execute("SELECT num FROM record WHERE name='dnsbl-total';")
			if num:
				self.DNSBL_TOTAL = self.dbp.fetchone()[0]

			num = self.dbp.execute("SELECT num FROM record WHERE name='dnsbl-found';")
			if num:
				self.DNSBL_BANNED = self.dbp.fetchone()[0]

		except Exception, err:
			self.logger.log("Error fetching historical records for dnsbl module: %s" % err, L_ERROR)
		
		try:
			self.use_netstats = self.config.getint('dnsbl', 'use_netstats')
		except Exception, err:
			self.logger.log("Error, setting 'dnsbl:use_netstats' does not exist in config: %s" % err, L_ERROR)
			self.use_netstats = 0

		if self.use_netstats:
			try:
				self.dbn = db.connect(self.config.get('netstats', 'dsn'), isolation_level=None)
			except Exception, err:
				self.logger.log("Error, setting 'dnsbl:dsn' does not exist in config: %s" % err, L_ERROR)
				self.use_netstats = 0
				self.parent.privMsg(self.logchan,
					"dnsbl:use_netstats disabled due to invalid configuration", self.uid)
		
		return True
	
	def shutdown(self):
		PSModule.shutdown(self)
		try:
			
			self.dbp.execute("INSERT INTO record (name, num) VALUES ('dnsbl-total', %s) ON DUPLICATE KEY UPDATE num=%s;", (self.DNSBL_TOTAL,self.DNSBL_TOTAL))
			self.dbp.execute("INSERT INTO record (name, num) VALUES ('dnsbl-found', %s) ON DUPLICATE KEY UPDATE num=%s;", (self.DNSBL_BANNED,self.DNSBL_BANNED))
		except Exception, err:
			self.logger.log("Error saving historical records for dnsbl module: %s" % err, L_ERROR)
		
		try:
			if self.use_netstats:
				self.dbn.close()
		except Exception, err:
			self.logger.log("Error closing database handle for dnsbl-netstats integration: %s" % err, L_ERROR)
		
		self.parent.quitFakeUser(self.uid)

## Begin event hooks
	def dnsbl_CHECK(self, raw_ip):
		"""Check host for DNSBL
		Param is the ip to check
		"""

		r = raw_ip.split(".")
		try:
			rip = "%s.%s.%s.%s" % (r[3], r[2], r[1], r[0])
		except IndexError, err:
			self.logger.log("Invalid IP supplied to dnsbl_CHECK (%s)" % raw_ip, L_ERROR)
			return False
			
		for base_dns, reference in self.DNSBL_LIST:
			test = "%s.%s" % (rip, base_dns)
			try:
				self.logger.log("testing %s" % test, L_DEBUG)
				code = socket.gethostbyname(test).split(".")[3]
				self.DNSBL_BANNED += 1
				return (code, reference, base_dns)
			except Exception, err:
				self.logger.log("DNS_RESULT: %s did not resolve %s (%s)" % (base_dns, raw_ip, err),
					L_DEBUG)
		return False

	def dnsbl_HANDLE(self, cb_list, user, **kwargs):
		"""Callback for the main dnsbl checker function which
		is threaded for obvious reasons
		"""

		if not cb_list or not user: return

		self.parent.privMsg(self.logchan,
			"DNSBL: Banned %s!%s@%s Source: %s (Code #%s) (#%d/%d)" %
			(user['nick'], user['user'], user['ip'], cb_list[2], cb_list[0],
			self.DNSBL_BANNED, self.DNSBL_TOTAL), self.uid)
		if not self.parent.get_user("GeoServ"): serv = "OperServ"
		else: serv = "GeoServ"
		self.parent.privMsg(serv,
			"akill add +3d *@%s Blacklisted IP. Visit %s%s for more information. (Code #%s)" %
			(user['ip'], cb_list[1], user['ip'], cb_list[0]), self.uid)
			
	def dnsbl_UID(self, prefix, params):
		"""Called when a user signs on, starts the thread to check their host
		Optionally if specified in config, can skip check if user is in kline table
		TODO ^^ will be removed soon.
		"""
		
		self.DNSBL_TOTAL += 1
		if self.use_netstats:
			try:
				num = self.dbp.execute("SELECT * FROM kline WHERE ip=?;", (params[6],))
				if num:
					self.logger.log("debug: Passed scanning on %s because its in the kline table" % (params[6]), L_STATUS)
					return
			except Exception, err:
				self.logger.log("Error checking kline table in dnsbl_UID: %s" % err, L_ERROR)
		

		#self.parent.privMsg(self.logchan, "DNSBL: Checking %s!%s@%s (#%d)" %
		#	(params[0], params[4], params[6], self.DNSBL_TOTAL), self.uid)
		d = threads.deferToThread(self.dnsbl_CHECK, params[6])
		d.addCallback(self.dnsbl_HANDLE, self.parent.user_t[params[7]])
## End event hooks

## Begin Command hooks
	def cmd_reload(self, source, target, pieces):
		"""Try to reload the blacklists from config
		TODO: move this out of config
		"""
		
		if self.reload():
			self.parent.privMsg(target,
				"DNSBL: reload successful",
				self.uid)
		else:
			self.parent.privMsg(target,
				"DNSBL: reload failed, check your config file",
				self.uid)
		return True

	def cmd_dnscan(self, source, target, pieces):
		"""Scan the IP of the nickname and handoff"""

		if not pieces: return False

		uid, user = self.parent.get_user(pieces[0])
		if not user:
			self.parent.privMsg(target,
				"%s is not online.  Not scanning." % (pieces[0]), self.uid)
		else:
			d = threads.deferToThread(self.dnsbl_CHECK, user['ip'])
			d.addCallback(self.dnsbl_HANDLE, user)

		return True
## End Command hooks

	def getCommands(self):
		return (
			('dreload', {
			'master':None,
			'callback':self.cmd_reload,
			'usage':"- reloads the dnsbl list from the config file"}),
			('dnscan', {
			'channel':self.logchan,
			'callback':self.cmd_dnscan,
			'usage':"<nickname> - scans nickname and bans if necessary"}),
		)

	def getHooks(self):
		return (('uid', self.dnsbl_UID),)
		
