#!/usr/bin/python pseudoserver.py
#psm_dnsbl-admin.py
# module for pypseudoserver
# written by radicand
# THIS IS SPECIFIC TO THE RIZON DNSBL
# YOU CAN ADAPT IT TO YOURS IF YOU MAKE A COMPLIANT API

from psmodule import *
import urllib2 as u
import urllib as u1
import re

class PSModule_dnsbl_admin(PSModule):
	VERSION = 1
	
	uid = ""

	def __init__(self, parent, config):
		PSModule.__init__(self, parent, config)
		self.reload()

	def startup(self):
		if not PSModule.startup(self): return False
		
		self.uid = self.parent.uid

		return True

	def reload(self):
		PSModule.reload(self)
		try:
			self.api = self.config.get('dnsbl_admin', 'api')
			self.key = self.config.get('dnsbl_admin', 'key')
			self.klinechan = self.config.get('dnsbl_admin', 'kline_chan')
			self.dnsbl_url = self.config.get('dnsbl_admin', 'dnsbl_url')
		except Exception, err:
			self.logger.log("Error reading dnsbl_admin config.ini settings: %s" % err, L_ERROR)
			raise

		self.url = self.api + "/?key=" + self.key
	
	def add(self, ip, code, reason):
		"""External method to allow other modules to
		use for purposes of banning (e.g., BOPM discoveries)"""

		try:
			fp = u.urlopen("%s&op=add&val=%s" %
				(self.url, u1.quote("%s|%s|%s"
					% (ip, code, reason))))
		except u.URLError, err:
			self.logger.log("Error contacting dnsbl: %s" % err, L_ERROR)
			return False
		
		code = fp.read()
		
		if code == "200":
			self.logger.log("%s/%s/%s Added to DNSBL via add()" % (ip, code, reason), L_DEBUG)
			return True
		else:
			self.logger.log("%s/%s/%s add() failed: Err %s" % (ip, code, reason, code), L_ERROR)
			return False
		

	def cmd_search(self, source, target, pieces):
		"""Search the dnsbl for a matching ip string"""
		
		if not pieces: return False
		
		try:
			fp = u.urlopen("%s&op=search&ip=%s" % (self.url, pieces[0]))
		except u.URLError, err:
			self.logger.log("Error searching dnsbl: %s" % err, L_ERROR)
			self.parent.privMsg(target, "Error searching DNSBL, sorry.", self.uid)
			return True

		ret = re.findall(r"(\d+)\t([^\t]+)\t(\d+)\t([^\t]+)\t(.*)", fp.read())
		for m in ret:
			self.parent.notice(source, "#%s %s: %s (#%s) - %s" %
				(m[0], m[1], m[3], m[2], m[4]), self.uid)
		
		self.parent.notice(source, "END OF DSEARCH (%d/%d)" % (len(ret),min(20,len(ret))), self.uid)
		return True

	def cmd_add(self, source, target, pieces):
		"""Search the dnsbl for a matching ip string"""
		
		if not pieces: return False

		ret = self.add(pieces[0], pieces[1], " ".join(pieces[2:]))
		
		if not ret:	
			self.parent.privMsg(target, "Error contacting or adding to DNSBL, sorry.", self.uid)
			return True
		
		self.parent.privMsg(target, "%s Added." % pieces[0], self.uid)
		uid, user = self.parent.get_user("GeoServ") or self.parent.get_user("OperServ")
		self.parent.privMsg(uid, "AKILL ADD +3d *@%s Blacklisted IP. Visit %s%s for more information: Code %s: %s." %
			(pieces[0], self.dnsbl_url, pieces[0], pieces[1], " ".join(pieces[2:])), self.uid)
		
		return True
		
	def cmd_del(self, source, target, pieces):
		"""Search the dnsbl for a matching ip string"""
		
		if not pieces: return False
		
		try:
			fp = u.urlopen("%s&op=del&val=%s" % (self.url, u1.quote(pieces[0])))
		except u.URLError, err:
			self.logger.log("Error contacting dnsbl: %s" % err, L_ERROR)
			self.parent.privMsg(target, "Error contacting DNSBL, sorry.", self.uid)
			return True
		
		code = fp.read()
		self.logger.log("DNSBL_DEL Return Code Was: %s" % code, L_DEBUG)
		
		if code == "200":
			self.parent.privMsg(target, "%s deleted." % pieces[0], self.uid)
			uid, user = self.parent.get_user("GeoServ") or self.parent.get_user("OperServ")
			self.parent.privMsg(uid, "AKILL DEL *@%s" % pieces[0], self.uid)
		else:
			self.parent.privMsg(target, "Deletion failed: Err %s" % code, self.uid)
		
		return True

	def getCommands(self):
		return (
			('dsearch', {
			'channel':(self.logchan,self.klinechan),
			'callback':self.cmd_search,
			'usage':"<partial ip address> - searches and returns up to 20 matches from the dnsbl"}),
			('dadd', {
			'channel':(self.logchan,self.klinechan),
			'callback':self.cmd_add,
			'usage':"<ip address> <class=2|3|5> <reason> - add a ip to the dnsbl"}),
			('ddel', {
			'channel':(self.logchan,self.klinechan),
			'callback':self.cmd_del,
			'usage':"<ip address> - deletes an ip from the dnsbl"}),
		)
