#!/usr/bin/python
# A IRCD PseudoServer, written for Rizon
# (c) Nick Pappas (radicand) http://log.radicand.org/
# V2.0

PROFILE = False
if PROFILE:
	try:
		import cProfile as profiler
	except ImportError:
		try:
			import profile as profiler
		except ImportError:
			print "Unable to import any profilers, continuing with profiling disabled..."
			PROFILE = False

from twisted.internet import reactor
from twisted.internet.protocol import ClientFactory
from twisted.python import log
import sys, os
import ConfigParser

IRCF_COLOR = chr(3)

config = ConfigParser.ConfigParser()
config.read("config.ini")

#GLOBALS
L_IN = 1
L_OUT = 2
L_STATUS = 4
L_SECURITY = 8
L_DEBUG = 16
L_SQL = 32
L_COMMAND = 64
L_ERROR = 128

### ### ### ###### ### ### ###### ### ### ###### ### ### ###### ### ### ###
class MessageLogger(object):
	"""An independent logger class (because separation of application
	and protocol logic is a good thing).
	"""
	loglevel = config.getint('logging', 'level')

	def log(self, message, level):
		if not self.loglevel&level == level: return
		"""Write a message to the file."""
		print message.encode('utf-8')
		
	def close(self):
		pass
		
class PseudoServerFactory(ClientFactory):
	"""A PseudoServer Factory class
	"""
	
	def __init__(self):
		self.logger = MessageLogger()
		try:
			prototype = config.get('server', 'protocol')
			proto = __import__("protocol." + prototype.lower())
			self.protocol = eval(
				"proto." + prototype.lower() + "." + prototype)
		except Exception, err:
			self.logger.log("Error loading protocol '%s'. (%s)" % (prototype, err), L_ERROR)
			sys.exit(1)
		
	def startedConnecting(self, connector):
		self.logger.log('Started to connect.', L_STATUS)
		
	def clientConnectionLost(self, connector, reason):
		self.logger.log('Lost connection.  Reason: %s' % reason, L_STATUS)
		try:
			reactor.stop()
		except Exception, err:
			self.logger.log("Tried to stop the reactor: connection lost (%s)" % err, L_ERROR)

		
	def clientConnectionFailed(self, connector, reason):
		self.logger.log('Connection failed.  Reason: %s' % reason, L_STATUS)
		try:
			reactor.stop()
		except Exception, err:
			self.logger.log("Tried to stop the reactor: connection failed (%s)" % err, L_ERROR)

#core startup functionality
def main():
	#init logger
	
	if config.getint('control', 'debug') == 1:
		logfile = sys.stdout
	else:
		flog = config.get('logging', 'logfile')
		logfile = open(flog, 'w')
	log.startLogging(logfile)

	# increase the threadpool
	try:
		threads = config.getint('server', 'threads')
		if threads <= 0:
			raise Exception("server threads must be greater than 0")
	except ValueError:
		print "Error, server threads parameter in config was malformed.  Please check it."
	except Exception, err:
		print "Error, %s" % err
	
	reactor.suggestThreadPoolSize(threads)
	
	#create factory
	fac = PseudoServerFactory()
	
	#connect
	reactor.connectTCP(
		config.get('uplink', 'ip'),
		config.getint('uplink', 'port'),
		fac)
	
	#go for it.
	reactor.run()
	
if __name__ == "__main__":
	try:
		if not PROFILE:
			import psyco
			psyco.full()
	except Exception, err:
		print "Error initializing psyco optimizer, continuing without it...: %s" % err

	if config.getint('control', 'debug') == 1:
		if PROFILE: profiler.run('main()', 'profile.log')
		else: main()
	else:
		try:
			pid = os.fork()
			if pid > 0: sys.exit(0)
		except OSError, err:
			print "Error #%d encountered while trying to fork: %s" % (err.errno, err.strerror)
			sys.exit(1)
		
		try:
			pid = os.fork()
			if pid > 0:
				print "Daemonizing into PID %d" % pid
				sys.exit(0)
		except OSError, err:
			print "Error #%d encountered while trying to do second fork: %s" % (err.errno, err.strerror)
			sys.exit(1)

		main()
	
